package data

import data.Clustering.{DimId, DpId, SubClusId}

/**
  * A subspace cluster which is defined as a set of features, a set of data points and an identifier
  *
  * The equals method is overridden in order to allow only referential equality
  */
case class SubspaceCluster(id: SubClusId, dimensions: Set[DimId], dataPoints: Set[DpId])
{
  lazy val size = dataPoints.size

  override def equals(obj: scala.Any): Boolean = obj match {
    case x: AnyRef => this eq x
    case _ => false
  }
}

/**
  * ConsensusSubspaceCluster cluster is a meta cluster which is defined as a set of features, a set of data points,
  * an identifier and the number of clusters.
  *
  * The equals method is overridden in order to allow only referential equality
  */
case class ConsensusSubspaceCluster(rowGroupId: Int, nrOfClusters: Int, dimensions: Set[Int], dataPoints: Set[DpId])
{
   def size = dataPoints.size

  override def equals(obj: scala.Any): Boolean = obj match {
    case x: AnyRef => this eq x
    case _ => false
  }
}


/**
  * A simple data point class containing an identifier,
  * a cluster label which the data point belongs to and the features
 */
case class DataPoint(id: DpId,
                     clusterId: Int,
                     features: IndexedSeq[Double])

/**
  * Some Type alias for the identifiers
 */
object Clustering {
  type DpId = Int
  type DimId = Int
  type SubClusId = Int
}