package data

import better.files._
import scala.collection.immutable.BitSet
import scala.collection.mutable.ListBuffer

/**
  * Just a small helper object which parses  files into datasets and subspace clusters
  */
object DataIO {

  /**
    * This function loads a subspace clustering outcome contained in the given file.
    * Each cluster is represented by three lines. the first contains a cluster identifier (only unique for this result)
    * The second line contains the data point ids assigned to this cluster
    * the third line contains the feature ids assigned to this cluster
    */
  def loadClustering(file: File): Seq[SubspaceCluster] = {
    try {

      val clusters = new ListBuffer[SubspaceCluster]
      for {
        reader <- file.bufferedReader
      } {
        while (reader.ready()) {
          val clusterid = reader.readLine().toInt
          val dimensions = BitSet(reader.readLine().split(',').map(_.toInt): _*)
          val datapoints = reader.readLine().split(',').map(_.toInt).toSet
          clusters.append(SubspaceCluster(clusterid, dimensions, datapoints))
        }
      }
      clusters.toVector
    } catch {
      case e: Exception =>
        println(file.pathAsString)
        throw e
    }
  }

  /**
    * Saves a given sequence of meta clusters to the given file.
    * The file format is similar to the one described in DataIO.loadClustering
    */
  def saveMetaClusters(metaClusters: Seq[ConsensusSubspaceCluster], file: File) = {
    file < metaClusters.zipWithIndex.map { case (mc, id) =>
      val dps = mc.dataPoints.toVector.sorted.mkString(",")
      val dims = mc.dimensions.toVector.sorted.mkString(",")
      s"$id\n$dps\n$dims"
    }.mkString("\n")
  }
}
