package example

import better.files._
import data.{DataIO, SubspaceCluster}
import subcluens.{GenerateDataTensor, SubCluEns}
import subcluens.datastructure.DataTensor

import scala.util.Random

/**
  * This small script shows how to execute SubCluEns at the example based on the Iris dataset (http://archive.ics.uci.edu/ml/datasets/Iris).
  * The directory exampleDataset/iris_consensus_data contains the outcome of 20 different subspace clustering algorithms
  * (Clique, Proclus, Subclu) with different parametrizations.
  * In the following we want to create a consensus subspace clustering based on these outcomes.
  */
object IrisExample extends App {

  //First we load the clustering outcomes for the consensus. Each file represents one clustering.
  //Each cluster is represented by three lines. the first contains a cluster identifier (only unique for this result)
  //The second line contains the data point ids assigned to this cluster
  //the third line contains the feature ids assigned to this cluster
  val clusterings: Seq[SubspaceCluster] = file"exampleDataset/iris_consensus_data".list.toSeq.flatMap { file =>
    DataIO.loadClustering(file)
  }
  //We create the triplets based on these clusters
  val (dataSet, _) = GenerateDataTensor.generateSet(clusterings)

  //We create from the triplets a initial DataTensor with only one group per dimension
  val initialTensor = DataTensor(dataSet)

  //We run SubCluEns which decomposes the tensor and creates the meta clusters
  val (metaClusters, encodingCosts) = SubCluEns.apply(initialTensor, Random.nextInt())
  //In case of the extension with a maximal allowed cluster group count we could add for instance optionalMaxClusterGroups = Some(3)

  //Last but not least, we save the result with the same schema as described above
  //We also add the encoding costs of the decomposed tensor
  DataIO.saveMetaClusters(metaClusters, file"exampleDataset/iris_consensus_$encodingCosts.data")
}
