package subcluens.operations

import subcluens.datastructure.EncodingCosts3d

/**
  * We use this class for book-keeping. It stores the intermediate steps during the split operation.
  *
  * @param nrOfMatrixElements The number of elements in each matrix-slice (a row element is a matrix based of columns and tubes)
  * @param currentNrOfRowElements The number of rows
  * @param currentNrOfTrues The current number of true elements in the cell (for each row which is removed this number is reduced)
  */
case class CurrentCellProps(nrOfMatrixElements: Int, //Fixed as only the number of row elements changes
                            currentNrOfRowElements: Int, //Can also be a column or tube depending on the context
                            currentNrOfTrues: Double) {
  val currentAvgEncodingCostsPerRow =
    EncodingCosts3d.determineAvgEncodingCostsPerEntity(currentNrOfTrues,
      nrOfMatrixElements * currentNrOfRowElements, currentNrOfRowElements)

  def removeRowElement(nrOfTruesInMatrix: Int) = {
    this.copy(currentNrOfRowElements = currentNrOfRowElements - 1, currentNrOfTrues = currentNrOfTrues - nrOfTruesInMatrix)
  }
}

