package subcluens.operations

import subcluens.datastructure.{CaCell, DataTensor}

import scala.math._

/**
  * Fuses the two most similar rows (based on the average distance between encoding costs of true-entries over all tensor groups)
  */
object FusionRows  {
   def performOperation[X, Y, Z](dm: DataTensor[X, Y, Z]): DataTensor[X, Y, Z] = {
    if (dm.nrOf1DGroups > 1) {
      val (idxA, idxB) = minEncodingPair(dm)
      val aIds = dm.getDim1Ids(idxA)
      val bIds = dm.getDim1Ids(idxB)
      val keep = min(idxA, idxB)
      val ignore = max(idxA, idxB)
      var currentIdxShift = 0
      val combinedIds = aIds ++ bIds

      val unsortedRows = new Array[IndexedSeq[IndexedSeq[CaCell[X, Y, Z]]]](dm.nrOf1DGroups - 1)
      for {
        rowIdx <- dm.dim1Indices
      } {
        if (rowIdx == keep) {
          unsortedRows(rowIdx) = dm.caCells(rowIdx).map(_.map { gt =>
            gt.copyWithNewDim1(dim1Ids = combinedIds)
          })
        } else if (rowIdx == ignore) {
          //Just ignore it from now on we have to shift the indices of all other rows by one
          currentIdxShift += 1
        } else {
          //All others
          unsortedRows(rowIdx - currentIdxShift) = dm.caCells(rowIdx)
        }
      }
      dm.copyWithNewCaCells(unsortedRows)
    } else {
      dm
    }
  }

  def minEncodingPair[X, Y, Z](dm: DataTensor[X, Y, Z]): (Int, Int) = {
    val best = (0 until dm.nrOf1DGroups).view.combinations(2).maxBy { case Seq(xIdx, yIdx) =>
      val x = dm.getCaCellsInDim1(xIdx)
      val y = dm.getCaCellsInDim1(yIdx)
      val xCosts = x.segmentEncodingCosts
      val yCosts = y.segmentEncodingCosts
      val combinedCosts = x.combinedSegmentEncodingCosts(y)
      (xCosts + yCosts) - combinedCosts
    }
    best(0) -> best(1)
  }
}