package subcluens.operations

import subcluens.datastructure.{CaCell, DataTensor}

/**
  * Implements the regroup operation for rows
  */
object ReGroupRows extends  AbstractRegroup {

  protected def determineEntryMatrixSize[X, Y, Z](ca: CaCell[X, Y, Z]): Int = ca.matrixSizeForDim1Idx

   def performOperation[X, Y, Z](dm: DataTensor[X, Y, Z]): DataTensor[X, Y, Z] = {
    val rowGroupedSlices = dm.dim1Indices.map(idx => dm.getCaCellsInDim1(idx))

    val idUpdates = regroupIds(rowGroupedSlices)

    if (idUpdates.isEmpty) {
      //Nothing changed we can return the old one
      dm
    } else {
      val updates = idUpdates.get
      var currentIdxShift = 0
      val newRowData = new Array[IndexedSeq[IndexedSeq[CaCell[X, Y, Z]]]](updates.size)
      for {
        rowGroupIdx <- dm.dim1Indices
      } {
        updates.get(rowGroupIdx).fold {
          currentIdxShift += 1 //This segment is empty after regrouping
        } { x =>
          newRowData(rowGroupIdx - currentIdxShift) = x.fold {
            dm.caCells(rowGroupIdx) //No updates
          } { newIds =>
            dm.caCells(rowGroupIdx).map(_.map { gt =>
              gt.copyWithNewDim1(dim1Ids = newIds)
            })
          }
        }
      }
      dm.copyWithNewCaCells(newRowData)
    }
  }
}
