package utils

import scala.math._

/**
  * Utility function for the logarithm with base two and log star two.
  */
object LogarithmUtils {

  val log2Base = log(2)

  /**
    * Calculates the optimal encoding costs of a integer number
    * As described in Universal Prior for Integers and Estimation by Minimum Description Length
    * Do not confuse it with https://en.wikipedia.org/wiki/Iterated_logarithm!
    * @param x
    */
  @inline
  def logStar2(x: Int): Double = {
    //We determine log(x) + log(log(x)) + log(...) + ...
    var sum = 0d
    var lastPart = log(x) / log2Base
    while (lastPart > 1) {
      sum += lastPart
      lastPart = log(lastPart) / log2Base
    }
    sum += lastPart
    sum
  }

  @inline
  def log2(x: Double): Double = log(x) / log2Base
}

